"""Tests that CRUD maven remotes."""

import json
import uuid

import pytest

import django

from pulpcore.client.pulp_maven.exceptions import ApiException

django.setup()
from pulp_maven.app.serializers import MavenRemoteSerializer  # noqa


@pytest.mark.parallel
def test_remote_crud_workflow(maven_remote_api_client, gen_object_with_cleanup, monitor_task):
    remote_data = {"name": str(uuid.uuid4()), "url": "http://example.com"}
    remote = gen_object_with_cleanup(maven_remote_api_client, remote_data)
    assert remote.url == remote_data["url"]
    assert remote.name == remote_data["name"]

    with pytest.raises(ApiException) as exc:
        gen_object_with_cleanup(maven_remote_api_client, remote_data)
    assert exc.value.status == 400
    assert json.loads(exc.value.body) == {"name": ["This field must be unique."]}

    update_response = maven_remote_api_client.partial_update(
        remote.pulp_href, {"url": "https://example.com"}
    )
    task = monitor_task(update_response.task)
    assert task.created_resources == []

    remote = maven_remote_api_client.read(remote.pulp_href)
    assert remote.url == "https://example.com"

    all_new_remote_data = {"name": str(uuid.uuid4()), "url": "http://example.com"}
    update_response = maven_remote_api_client.update(remote.pulp_href, all_new_remote_data)
    task = monitor_task(update_response.task)
    assert task.created_resources == []

    remote = maven_remote_api_client.read(remote.pulp_href)
    assert remote.name == all_new_remote_data["name"]
    assert remote.url == all_new_remote_data["url"]


@pytest.mark.parallel
def test_create_maven_remote_with_invalid_parameter():
    unexpected_field_remote_data = {
        "name": str(uuid.uuid4()),
        "url": "http://example.com",
        "foo": "bar",
    }

    maven_remote_serializer = MavenRemoteSerializer(data=unexpected_field_remote_data)

    assert maven_remote_serializer.is_valid() is False
    assert maven_remote_serializer.errors["foo"][0].title() == "Unexpected Field"


@pytest.mark.parallel
def test_create_maven_remote_without_url(maven_remote_api_client, gen_object_with_cleanup):
    maven_remote_serializer = MavenRemoteSerializer(data={"name": str(uuid.uuid4())})

    assert maven_remote_serializer.is_valid() is False
    assert maven_remote_serializer.errors["url"][0].title() == "This Field Is Required."


@pytest.mark.parallel
def test_default_remote_policy_immediate(maven_remote_api_client, gen_object_with_cleanup):
    remote_data = {"name": str(uuid.uuid4()), "url": "http://example.com"}
    remote = gen_object_with_cleanup(maven_remote_api_client, remote_data)
    assert remote.policy == "immediate"



from gettext import gettext as _

from rest_framework import serializers

from pulpcore.plugin import serializers as platform

from . import models


class MavenRepositorySerializer(platform.RepositorySerializer):
    """
    Serializer for Maven Repositories.
    """

    class Meta:
        fields = platform.RepositorySerializer.Meta.fields
        model = models.MavenRepository


class MavenArtifactSerializer(platform.SingleArtifactContentSerializer):
    """
    A Serializer for MavenArtifact.
    """

    group_id = serializers.CharField(
        help_text=_("Group Id of the artifact's package."), read_only=True
    )
    artifact_id = serializers.CharField(
        help_text=_("Artifact Id of the artifact's package."), read_only=True
    )
    version = serializers.CharField(
        help_text=_("Version of the artifact's package."), read_only=True
    )
    filename = serializers.CharField(help_text=_("Filename of the artifact."), read_only=True)

    class Meta:
        fields = platform.SingleArtifactContentSerializer.Meta.fields + (
            "group_id",
            "artifact_id",
            "version",
            "filename",
        )
        model = models.MavenArtifact


class MavenRemoteSerializer(platform.RemoteSerializer):
    """
    A Serializer for MavenRemote.

    Add any new fields if defined on MavenRemote.
    Similar to the example above, in MavenArtifactSerializer.
    Additional validators can be added to the parent validators list

    For example::

    class Meta:
        validators = platform.RemoteSerializer.Meta.validators + [myValidator1, myValidator2]
    """

    class Meta:
        fields = platform.RemoteSerializer.Meta.fields
        model = models.MavenRemote


class MavenDistributionSerializer(platform.DistributionSerializer):
    """
    Serializer for Maven Distributions.
    """

    remote = platform.DetailRelatedField(
        required=False,
        help_text=_("Remote that can be used to fetch content when using pull-through caching."),
        queryset=models.MavenRemote.objects.all(),
        view_name="remotes-maven/maven-detail",
        allow_null=True,
    )

    class Meta:
        fields = platform.DistributionSerializer.Meta.fields + ("remote",)
        model = models.MavenDistribution


class RepositoryAddCachedContentSerializer(platform.ValidateFieldsMixin, serializers.Serializer):
    remote = platform.DetailRelatedField(
        required=False,
        view_name_pattern=r"remotes(-.*/.*)-detail",
        queryset=models.Remote.objects.all(),
        help_text=_(
            "A remote to use to identify content that was cached. This will override a "
            "remote set on repository."
        ),
    )

    def validate(self, data):
        data = super().validate(data)
        repository = None
        if "repository_pk" in self.context:
            repository = models.Repository.objects.get(pk=self.context["repository_pk"])
        remote = data.get("remote", None) or getattr(repository, "remote", None)

        if not remote:
            raise serializers.ValidationError(
                {"remote": _("This field is required since a remote is not set on the repository.")}
            )
        self.check_cross_domains({"repository": repository, "remote": remote})
        return data



import hashlib
import re
from tempfile import NamedTemporaryFile

from django.conf import settings
from django.shortcuts import get_object_or_404, redirect
from django.db import IntegrityError

from rest_framework.exceptions import Throttled
from rest_framework.views import APIView
from rest_framework.response import Response

from pulp_maven.app.models import (
    MavenArtifact,
    MavenMetadata,
    MavenRepository,
    MavenDistribution,
)
from pulpcore.plugin.models import Artifact, ContentArtifact
from pulpcore.plugin.tasking import add_and_remove, dispatch


def has_task_completed(task):
    """
    Verify whether an immediate task ran properly.

    Returns:
        bool: True if the task ended successfully.

    Raises:
        Exception: If an error occured during the task's runtime.
        Throttled: If the task did not run due to resource constraints.

    """
    if task.state == "completed":
        task.delete()
        return True
    elif task.state == "canceled":
        raise Throttled()
    else:
        error = task.error
        task.delete()
        raise Exception(str(error))


class PomResponse(Response):
    def __init__(self, pom_artifact, status=200):
        artifact = pom_artifact._artifacts.get()
        size = artifact.size
        headers = {
            "Content-Type": "application/xml",
            "Content-Length": size,
            "ETag": f"{{SHA1}}{artifact.sha1}",
        }
        super().__init__(headers=headers, status=status)


class MavenApiViewSet(APIView):
    """
    ViewSet for interacting with maven deploy API
    """

    model = MavenRepository
    queryset = MavenRepository.objects.all()

    lookup_field = "name"

    # Authentication disabled for now
    authentication_classes = []
    permission_classes = []

    def redirect_to_content_app(self, distribution, relative_path):
        return redirect(
            f"{settings.CONTENT_ORIGIN}{settings.CONTENT_PATH_PREFIX}{distribution.base_path}/"
            f"{relative_path}"
        )

    def get_repository_and_distributions(self, name):
        repository = get_object_or_404(MavenRepository, name=name)
        distribution = get_object_or_404(MavenDistribution, repository=repository)
        return repository, distribution

    @staticmethod
    def is_metadata(path):
        is_metadata = False
        pattern = r"\.(xml|xml\.sha1|xml\.md5|xml\.sha224|xml\.sha256|xml\.sha384|xml\.sha512)$"
        if re.search(pattern, path):
            is_metadata = True
        return is_metadata

    @staticmethod
    def maven_artifact_attrs_from_path(path):
        group_id, artifact_id, version, name = MavenArtifact.group_artifact_version_filename(path)

        attrs = dict(
            filename=name,
            version=version,
            artifact_id=artifact_id,
            group_id=group_id,
        )
        return attrs

    def get(self, request, name, path):
        """
        Responds to GET requests about manifests by reference
        """
        repo, distro = self.get_repository_and_distributions(name)

        kwargs = self.maven_artifact_attrs_from_path(path)
        kwargs["pk__in"] = repo.latest_version().content

        if self.is_metadata(path):
            model = MavenMetadata
        else:
            model = MavenArtifact
        content = get_object_or_404(model, **kwargs)
        relative_path = content.contentartifact_set.get().relative_path
        return self.redirect_to_content_app(distro, relative_path)

    def put(self, request, name, path):
        repo, distro = self.get_repository_and_distributions(name)

        # Determine if this is an artifact or metadata
        is_metadata = self.is_metadata(path)

        # Save the uploaded file as an artifact
        chunk = request.META["wsgi.input"]
        artifact = self.receive_artifact(chunk)

        # Create a MavenArtifact or MavenMetadata
        if is_metadata:
            content = MavenMetadata.init_from_artifact_and_relative_path(artifact, path)
        else:
            content = MavenArtifact.init_from_artifact_and_relative_path(artifact, path)
        try:
            content.save()
        except IntegrityError:
            if is_metadata:
                content = MavenMetadata.objects.get(sha256=content.sha256)
            else:
                content = MavenArtifact.objects.get(
                    group_id=content.group_id,
                    artifact_id=content.artifact_id,
                    version=content.version,
                    filename=content.filename,
                )
        ca = ContentArtifact(artifact=artifact, content=content, relative_path=path)
        try:
            ca.save()
        except IntegrityError:
            ca = ContentArtifact.objects.get(content=content, relative_path=path)
            if not ca.artifact:
                ca.artifact = artifact
                ca.save()

        if is_metadata:
            metadata_to_remove = MavenMetadata.objects.filter(
                pk__in=repo.latest_version().content.all(),
                group_id=content.group_id,
                artifact_id=content.artifact_id,
                version=content.version,
                filename=content.filename,
            )
            remove_content_units = [str(c[0]) for c in metadata_to_remove.values_list("pk")]
        else:
            remove_content_units = []

        add_content_units = [str(content.pk)]

        dispatched_task = dispatch(
            add_and_remove,
            exclusive_resources=[repo],
            immediate=True,
            deferred=False,
            kwargs={
                "repository_pk": str(repo.pk),
                "add_content_units": add_content_units,
                "remove_content_units": remove_content_units,
            },
        )

        if has_task_completed(dispatched_task):
            return Response(status=201)

    @staticmethod
    def receive_artifact(chunk):
        """Handles assembling of a file as it's being uploaded."""
        with NamedTemporaryFile("ab") as temp_file:
            size = 0
            hashers = {}
            for algorithm in Artifact.DIGEST_FIELDS:
                hashers[algorithm] = getattr(hashlib, algorithm)()
            while True:
                subchunk = chunk.read(2000000)
                if not subchunk:
                    break
                temp_file.write(subchunk)
                size += len(subchunk)
                for algorithm in Artifact.DIGEST_FIELDS:
                    hashers[algorithm].update(subchunk)
            temp_file.flush()
            digests = {}
            for algorithm in Artifact.DIGEST_FIELDS:
                digests[algorithm] = hashers[algorithm].hexdigest()
            artifact = Artifact(file=temp_file.name, size=size, **digests)
            try:
                artifact.save()
            except IntegrityError:
                artifact = Artifact.objects.get(sha256=artifact.sha256)
                artifact.touch()
            return artifact



from django.db import migrations, models, transaction
import django.db.models.deletion


def migrate_data_from_old_model_to_new_model_up(apps, schema_editor):
    """ Move objects from MavenDistribution to NewMavenDistribution."""
    MavenDistribution = apps.get_model('maven', 'MavenDistribution')
    NewMavenDistribution = apps.get_model('maven', 'NewMavenDistribution')
    for maven_distribution in MavenDistribution.objects.all():
        with transaction.atomic():
            NewMavenDistribution(
                pulp_id=maven_distribution.pulp_id,
                pulp_created=maven_distribution.pulp_created,
                pulp_last_updated=maven_distribution.pulp_last_updated,
                pulp_type=maven_distribution.pulp_type,
                name=maven_distribution.name,
                base_path=maven_distribution.base_path,
                content_guard=maven_distribution.content_guard,
                remote=maven_distribution.remote,
                repository_version=maven_distribution.repository_version,
                repository=maven_distribution.repository
            ).save()
            maven_distribution.delete()


def migrate_data_from_old_model_to_new_model_down(apps, schema_editor):
    """ Move objects from NewMavenDistribution to MavenDistribution."""
    MavenDistribution = apps.get_model('maven', 'MavenDistribution')
    NewMavenDistribution = apps.get_model('maven', 'NewMavenDistribution')
    for maven_distribution in NewMavenDistribution.objects.all():
        with transaction.atomic():
            MavenDistribution(
                pulp_id=maven_distribution.pulp_id,
                pulp_created=maven_distribution.pulp_created,
                pulp_last_updated=maven_distribution.pulp_last_updated,
                pulp_type=maven_distribution.pulp_type,
                name=maven_distribution.name,
                base_path=maven_distribution.base_path,
                content_guard=maven_distribution.content_guard,
                remote=maven_distribution.remote,
                repository_version=maven_distribution.repository_version,
                repository=maven_distribution.repository
            ).save()
            maven_distribution.delete()


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ('maven', '0003_mavenrepository'),
    ]

    operations = [
        migrations.CreateModel(
            name='NewMavenDistribution',
            fields=[
                ('distribution_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='maven_mavendistribution', serialize=False, to='core.Distribution')),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.distribution',),
        ),
        migrations.RunPython(
            code=migrate_data_from_old_model_to_new_model_up,
            reverse_code=migrate_data_from_old_model_to_new_model_down,
        ),
        migrations.DeleteModel(
            name='MavenDistribution',
        ),
        migrations.RenameModel(
            old_name='NewMavenDistribution',
            new_name='MavenDistribution',
        ),
    ]



# Generated by Django 4.2.10 on 2025-02-24 20:58

from django.db import migrations, models
import django.db.models.deletion
import pulpcore.app.util


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0126_remoteartifact_failed_at'),
        ('maven', '0006_alter_mavenartifact_content_ptr_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='mavenartifact',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='mavenmetadata',
            unique_together=set(),
        ),
        migrations.AddField(
            model_name='mavenartifact',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='mavenmetadata',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AlterUniqueTogether(
            name='mavenartifact',
            unique_together={('group_id', 'artifact_id', 'version', 'filename', '_pulp_domain')},
        ),
        migrations.AlterUniqueTogether(
            name='mavenmetadata',
            unique_together={('group_id', 'artifact_id', 'version', 'filename', 'sha256', '_pulp_domain')},
        ),
    ]



from drf_spectacular.utils import extend_schema
from rest_framework.decorators import action

from pulpcore.plugin.serializers import AsyncOperationResponseSerializer

from pulpcore.plugin.tasking import dispatch

from pulpcore.plugin.viewsets import (
    ContentFilter,
    ContentViewSet,
    DistributionViewSet,
    OperationPostponedResponse,
    RemoteViewSet,
    RepositoryVersionViewSet,
    RepositoryViewSet,
)

from pulp_maven.app.models import MavenArtifact, MavenRemote, MavenRepository, MavenDistribution

from pulp_maven.app.serializers import (
    MavenArtifactSerializer,
    MavenRemoteSerializer,
    MavenRepositorySerializer,
    MavenDistributionSerializer,
    RepositoryAddCachedContentSerializer,
)

from pulp_maven.app.tasks import add_cached_content_to_repository


class MavenArtifactFilter(ContentFilter):
    """
    FilterSet for MavenArtifact.
    """

    class Meta:
        model = MavenArtifact
        fields = ["group_id", "artifact_id", "version", "filename"]


class MavenArtifactViewSet(ContentViewSet):
    """
    A ViewSet for MavenArtifact.
    """

    endpoint_name = "artifact"
    queryset = MavenArtifact.objects.all()
    serializer_class = MavenArtifactSerializer
    filterset_class = MavenArtifactFilter


class MavenRemoteViewSet(RemoteViewSet):
    """
    A ViewSet for MavenRemote.
    """

    endpoint_name = "maven"
    queryset = MavenRemote.objects.all()
    serializer_class = MavenRemoteSerializer


class MavenRepositoryViewSet(RepositoryViewSet):
    """
    A ViewSet for MavenRemote.
    """

    endpoint_name = "maven"
    queryset = MavenRepository.objects.all()
    serializer_class = MavenRepositorySerializer

    @extend_schema(
        description="Trigger an asynchronous task to add cached content to a repository.",
        summary="Add cached content",
        responses={202: AsyncOperationResponseSerializer},
    )
    @action(detail=True, methods=["post"], serializer_class=RepositoryAddCachedContentSerializer)
    def add_cached_content(self, request, pk):
        """
        Add to the repository any MavenArtifact and MavenMetadata that was cached using the
        remote since the last repository version was created.

        The ``repository`` field has to be provided.
        """
        serializer = RepositoryAddCachedContentSerializer(
            data=request.data, context={"request": request, "repository_pk": pk}
        )
        serializer.is_valid(raise_exception=True)

        repository = self.get_object()
        remote = serializer.validated_data.get("remote", repository.remote)

        result = dispatch(
            add_cached_content_to_repository,
            shared_resources=[remote],
            exclusive_resources=[repository],
            kwargs={
                "remote_pk": str(remote.pk),
                "repository_pk": str(repository.pk),
            },
        )
        return OperationPostponedResponse(result, request)


class MavenRepositoryVersionViewSet(RepositoryVersionViewSet):
    """
    MavenRepositoryVersion represents a single Maven repository version.
    """

    parent_viewset = MavenRepositoryViewSet


class MavenDistributionViewSet(DistributionViewSet):
    """
    ViewSet for Maven Distributions.
    """

    endpoint_name = "maven"
    queryset = MavenDistribution.objects.all()
    serializer_class = MavenDistributionSerializer



# Generated by Django 4.2.1 on 2023-05-17 07:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0106_alter_artifactdistribution_distribution_ptr_and_more"),
        ("maven", "0005_mavenmetadata"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mavenartifact",
            name="content_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="core.content",
            ),
        ),
        migrations.AlterField(
            model_name="mavendistribution",
            name="distribution_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="core.distribution",
            ),
        ),
        migrations.AlterField(
            model_name="mavenmetadata",
            name="content_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="core.content",
            ),
        ),
        migrations.AlterField(
            model_name="mavenremote",
            name="remote_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="core.remote",
            ),
        ),
        migrations.AlterField(
            model_name="mavenrepository",
            name="repository_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="core.repository",
            ),
        ),
    ]



import uuid

import pytest

from pulpcore.client.pulp_maven import (
    ApiClient,
    ContentArtifactApi,
    DistributionsMavenApi,
    RemotesMavenApi,
    RepositoriesMavenApi,
)


@pytest.fixture(scope="session")
def maven_client(_api_client_set, bindings_cfg):
    api_client = ApiClient(bindings_cfg)
    _api_client_set.add(api_client)
    yield api_client
    _api_client_set.remove(api_client)


@pytest.fixture(scope="session")
def maven_artifact_api_client(maven_client):
    return ContentArtifactApi(maven_client)


@pytest.fixture(scope="session")
def maven_distro_api_client(maven_client):
    return DistributionsMavenApi(maven_client)


@pytest.fixture(scope="session")
def maven_repo_api_client(maven_client):
    return RepositoriesMavenApi(maven_client)


@pytest.fixture(scope="session")
def maven_remote_api_client(maven_client):
    return RemotesMavenApi(maven_client)


@pytest.fixture
def maven_distribution_factory(maven_distro_api_client, gen_object_with_cleanup):
    def _maven_distribution_factory(**kwargs):
        data = {"base_path": str(uuid.uuid4()), "name": str(uuid.uuid4())}
        data.update(kwargs)
        return gen_object_with_cleanup(maven_distro_api_client, data)

    return _maven_distribution_factory


@pytest.fixture
def maven_repo_factory(maven_repo_api_client, gen_object_with_cleanup):
    """A factory to generate a Maven Repository with auto-deletion after the test run."""

    def _maven_repo_factory(**kwargs):
        kwargs.setdefault("name", str(uuid.uuid4()))
        return gen_object_with_cleanup(maven_repo_api_client, kwargs)

    yield _maven_repo_factory


@pytest.fixture
def maven_remote_factory(maven_remote_api_client, gen_object_with_cleanup):
    """A factory to generate a Maven Remote with auto-deletion after the test run."""

    def _maven_remote_factory(**kwargs):
        kwargs.setdefault("name", str(uuid.uuid4()))
        return gen_object_with_cleanup(maven_remote_api_client, kwargs)

    yield _maven_remote_factory



from pulpcore.plugin import PulpPluginAppConfig


class PulpMavenPluginAppConfig(PulpPluginAppConfig):
    """Entry point for the maven plugin."""

    name = "pulp_maven.app"
    label = "maven"
    version = "0.9.0"
    python_package_name = "pulp-maven"
    domain_compatible = True



import datetime

from pulp_maven.app.models import MavenRemote, MavenRepository

from pulpcore.plugin.models import Content, ContentArtifact, RemoteArtifact


def add_cached_content_to_repository(repository_pk=None, remote_pk=None):
    """
    Create a new repository version by adding content that was cached by pulpcore-content when
    streaming it from a remote.

    Args:
        repository_pk (uuid): The primary key for a Repository for which a new Repository Version
            should be created.
        remote_pk (uuid): The primary key for a Remote which will be used to identify Content
            created by pulpcore-content when it streamed it to clients.
    """
    repository = MavenRepository.objects.get(pk=repository_pk)
    remote = MavenRemote.objects.get(pk=remote_pk)

    latest_version = repository.latest_version()

    if latest_version.number == 0:
        date_min = datetime.datetime(1970, 1, 1, tzinfo=datetime.timezone.utc)
    else:
        date_min = latest_version.pulp_created
    with repository.new_version(base_version=None) as new_version:
        ca_id_list = RemoteArtifact.objects.filter(
            remote=remote, pulp_created__gte=date_min
        ).values_list("content_artifact")
        content_list = ContentArtifact.objects.filter(pk__in=ca_id_list).values_list("content")
        new_version.add_content(Content.objects.filter(pk__in=content_list))



# coding=utf-8
"""Utilities for tests for the maven plugin."""
import aiohttp
import asyncio
from dataclasses import dataclass


@dataclass
class Download:
    """Class for representing a downloaded file."""

    body: bytes
    response_obj: aiohttp.ClientResponse

    def __init__(self, body, response_obj):
        self.body = body
        self.response_obj = response_obj


def download_file(url, auth=None, headers=None):
    """Download a file.

    :param url: str URL to the file to download
    :param auth: `aiohttp.BasicAuth` containing basic auth credentials
    :param headers: dict of headers to send with the GET request
    :return: Download
    """
    return asyncio.run(_download_file(url, auth=auth, headers=headers))


async def _download_file(url, auth=None, headers=None):
    async with aiohttp.ClientSession(auth=auth, raise_for_status=True) as session:
        async with session.get(url, verify_ssl=False, headers=headers) as response:
            return Download(body=await response.read(), response_obj=response)



import os
import pytest
import subprocess
from urllib.parse import urljoin
from pulp_maven.tests.functional.utils import download_file


def test_mvn_deploy_workflow(
    maven_repo_api_client, maven_repo_factory, maven_distribution_factory, tmp_path, pulp_settings
):
    if pulp_settings.DOMAIN_ENABLED:
        pytest.skip("Maven deploy API does not currently work with domains enabled.")
    # Create a repository and distribution pointing to that repository.
    repo = maven_repo_factory()
    maven_distribution_factory(repository=repo.pulp_href)
    assert repo.latest_version_href.endswith("/versions/0/")

    # Deploy the simple project into the snapshot repository
    current_dir = os.path.dirname(os.path.abspath(__file__))
    try:
        # Copy the simple-project to a temporary directory to ensure proper permissions
        subprocess.check_output(
            ["cp", "-r", f"{current_dir}/../../assets/simple-project", f"{tmp_path}/simple-project"]
        )
        # Update pom.xml to point the Snapshots repository to the test repository
        subprocess.check_output(
            ["sed", "-i", f"s/maven-snapshots/{repo.name}/g", f"{tmp_path}/simple-project/pom.xml"]
        )
        # Run mvn deploy
        subprocess.run(
            ["mvn", "deploy"],
            cwd=f"{tmp_path}/simple-project",
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
    except subprocess.CalledProcessError as e:
        # The command had a non-zero exit code
        msg = e.stdout.decode() + e.stderr.decode()
        pytest.fail(msg)

    # Assert that the latest version is 12
    repo = maven_repo_api_client.read(repo.pulp_href)
    assert repo.latest_version_href.endswith("/versions/12/")

    # Assert that you can get the metadata for the simple-project
    pulp_unit_url = urljoin(
        "http://localhost:24817",
        f"/pulp/maven/{repo.name}/org/sonatype/nexus/examples/simple-project/maven-metadata.xml",
    )
    downloaded_file = download_file(pulp_unit_url)
    assert downloaded_file.response_obj.status == 200

    # Assert that a GET to the Maven API redirects to the content app
    assert not downloaded_file.response_obj.real_url.path.startswith("/pulp/maven")



# Generated by Django 3.2.18 on 2023-03-11 10:08

from django.db import migrations, models
import django.db.models.deletion
import pulp_maven.app.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0098_pulp_labels'),
        ('maven', '0004_swap_distribution_models'),
    ]

    operations = [
        migrations.CreateModel(
            name='MavenMetadata',
            fields=[
                ('content_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='maven_mavenmetadata', serialize=False, to='core.content')),
                ('group_id', models.CharField(max_length=255)),
                ('artifact_id', models.CharField(max_length=255)),
                ('version', models.CharField(max_length=255, null=True)),
                ('filename', models.CharField(max_length=255)),
                ('sha256', models.CharField(db_index=True, max_length=64, unique=True)),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
                'unique_together': {('group_id', 'artifact_id', 'version', 'filename', 'sha256')},
            },
            bases=(pulp_maven.app.models.MavenContentMixin, 'core.content'),
        ),
    ]



from gettext import gettext as _
from logging import getLogger
from os import path
import re

from django.db import models

from pulpcore.plugin.models import Content, Remote, Repository, Distribution
from pulpcore.plugin.util import get_domain_pk

logger = getLogger(__name__)


class MavenContentMixin:
    @staticmethod
    def group_artifact_version_filename(relative_path):
        """
        Converts a relative path into a tuple of group_id, artifact_id, and version.

        Args:
            relative_path (str): Relative path for the artifact in the repository.

        Returns:
            Tuple (group_id, artifact_id, version, filename)

        """
        sub_path, filename = path.split(relative_path)
        sub_path, version = path.split(sub_path)
        pattern = re.compile(r"\d+(\.\d+)?(\.\d+)?([.-][a-zA-Z0-9]+)*")
        if pattern.match(version) is None:
            artifact_id = version
            version = None
            group_id = sub_path.replace("/", ".")
        else:
            sub_path, artifact_id = path.split(sub_path)
            group_id = sub_path.replace("/", ".")

        return group_id, artifact_id, version, filename


class MavenArtifact(MavenContentMixin, Content):
    """
    The Maven artifact content type.

    This content type represents a single file in a Maven repository.
    """

    TYPE = "artifact"

    _pulp_domain = models.ForeignKey("core.Domain", default=get_domain_pk, on_delete=models.PROTECT)
    group_id = models.CharField(max_length=255, null=False)
    artifact_id = models.CharField(max_length=255, null=False)
    version = models.CharField(max_length=255, null=False)
    filename = models.CharField(max_length=255, null=False)

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"
        unique_together = ("group_id", "artifact_id", "version", "filename", "_pulp_domain")

    @staticmethod
    def init_from_artifact_and_relative_path(artifact, relative_path):
        """
        Returns an instance of MavenArtifact for this artifact.

        Args:
            artifact (:class:`~pulpcore.plugin.models.Artifact`): An instance of an Artifact
            relative_path (str): Relative path for the artifact in the Project

        """
        if path.isabs(relative_path):
            raise ValueError(_("Relative path can't start with '/'."))

        group_id, artifact_id, version, f_name = MavenArtifact.group_artifact_version_filename(
            relative_path
        )

        return MavenArtifact(
            group_id=group_id, artifact_id=artifact_id, version=version, filename=f_name
        )


class MavenMetadata(MavenContentMixin, Content):
    """
    The Maven Metadata content type.

    This content type represents a pom file or a pom.<checksum_type> file in a Maven repository.
    """

    TYPE = "metadata"

    _pulp_domain = models.ForeignKey("core.Domain", default=get_domain_pk, on_delete=models.PROTECT)
    group_id = models.CharField(max_length=255, null=False)
    artifact_id = models.CharField(max_length=255, null=False)
    version = models.CharField(max_length=255, null=True)
    filename = models.CharField(max_length=255, null=False)
    sha256 = models.CharField(max_length=64, null=False, unique=True, db_index=True)

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"
        unique_together = (
            "group_id",
            "artifact_id",
            "version",
            "filename",
            "sha256",
            "_pulp_domain",
        )

    @staticmethod
    def init_from_artifact_and_relative_path(artifact, relative_path):
        """
        Returns an instance of MavenMetadata for this artifact.

        Args:
            artifact (:class:`~pulpcore.plugin.models.Artifact`): An instance of an Artifact
            relative_path (str): Relative path for the artifact in the Project

        """
        if path.isabs(relative_path):
            raise ValueError(_("Relative path can't start with '/'."))

        group_id, artifact_id, version, f_name = MavenMetadata.group_artifact_version_filename(
            relative_path
        )

        return MavenMetadata(
            group_id=group_id,
            artifact_id=artifact_id,
            version=version,
            filename=f_name,
            sha256=artifact.sha256,
        )


class MavenRemote(Remote):
    """
    A Remote for MavenArtifact.

    Define any additional fields for your new importer if needed.
    """

    TYPE = "maven"

    @staticmethod
    def get_remote_artifact_content_type(relative_path=None):
        """
        Returns content type that is found at the relative_path.
        """
        pattern = r"\.(xml|xml\.sha1|xml\.md5|xml\.sha224|xml\.sha256|xml\.sha384|xml\.sha512)$"
        if re.search(pattern, relative_path):
            return MavenMetadata
        return MavenArtifact

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class MavenDistribution(Distribution):
    """
    Distribution for 'maven' content.
    """

    TYPE = "maven"

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class MavenRepository(Repository):
    """
    Repository for "maven" content.
    """

    TYPE = "maven"
    CONTENT_TYPES = [MavenArtifact, MavenMetadata]
    REMOTE_TYPES = [MavenRemote]

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"



"""Tests that verify download of content served by Pulp."""

from aiohttp.client_exceptions import ClientResponseError
import hashlib
from urllib.parse import urljoin

from pulp_maven.tests.functional.utils import download_file


def test_download_content(
    maven_distribution_factory,
    maven_remote_factory,
    maven_repo_factory,
    maven_artifact_api_client,
    maven_distro_api_client,
    maven_repo_api_client,
    monitor_task,
    distribution_base_url,
):
    """Verify whether content served by pulp can be downloaded.

    The process of creating a Maven mirror is:

    1. Create a Maven Remote with a URL pointing to the root of a Maven repository.
    2. Create a distribution with the remote set HREF from 1.

    Do the following:

    1. Create a Maven Remote and a Distribution.
    2. Select a random content unit in the distribution. Download that
       content unit from Pulp, and verify that the content unit has the
       same checksum when fetched directly from Maven Central.
    """
    remote = maven_remote_factory(url="https://repo1.maven.org/maven2/")
    repository = maven_repo_factory(remote=remote.pulp_href)
    distribution = maven_distribution_factory(
        remote=remote.pulp_href, repository=repository.pulp_href
    )

    # Pick a content unit, and download it from the remote repository
    unit_path = "academy/alex/custommatcher/1.0/custommatcher-1.0-javadoc.jar.sha1"
    remote_unit_url = urljoin(remote.url, unit_path)
    downloaded_file = download_file(remote_unit_url)
    remote_unit_checksum = hashlib.sha256(downloaded_file.body).hexdigest()

    # And from Pulp
    pulp_unit_url = urljoin(distribution_base_url(distribution.base_url), unit_path)
    downloaded_file = download_file(pulp_unit_url)
    pulp_unit_checksum = hashlib.sha256(downloaded_file.body).hexdigest()

    assert remote_unit_checksum == pulp_unit_checksum

    # Check that Pulp created a MavenArtifact
    content_response = maven_artifact_api_client.list(filename="custommatcher-1.0-javadoc.jar.sha1")
    assert content_response.count == 1

    # Remove the remote from the distribution
    monitor_task(
        maven_distro_api_client.partial_update(distribution.pulp_href, {"remote": None}).task
    )

    # Assert that the maven artifact is no longer available
    try:
        download_file(pulp_unit_url)
    except ClientResponseError as e:
        assert e.status == 404

    # Assert that the repository version is 0
    assert repository.latest_version_href.endswith("/versions/0/")

    # Add cached content to the repository
    monitor_task(maven_repo_api_client.add_cached_content(repository.pulp_href, {}).task)

    # Assert that the repository is at version 1
    repository = maven_repo_api_client.read(repository.pulp_href)
    assert repository.latest_version_href.endswith("/versions/1/")

    # Assert that it is now once again available from the same distribution
    downloaded_file = download_file(pulp_unit_url)
    assert downloaded_file.response_obj.status == 200
